from django.conf import settings

RECOMMENDS_STORAGE_MONGODB_DATABASE_DEFAULT = {
    'HOST': 'localhost',
    'PORT': 27017,
    'NAME': 'recommends',
}
RECOMMENDS_STORAGE_MONGODB_DATABASE = getattr(settings, 'RECOMMENDS_STORAGE_MONGODB_DATABASE', RECOMMENDS_STORAGE_MONGODB_DATABASE_DEFAULT)
RECOMMENDS_STORAGE_MONGODB_SIMILARITY_COLLECTION = getattr(settings, 'RECOMMENDS_STORAGE_MONGODB_SIMILARITY_COLLECTION', 'similarity')
RECOMMENDS_STORAGE_MONGODB_RECOMMENDATION_COLLECTION = getattr(settings, 'RECOMMENDS_STORAGE_MONGODB_RECOMMENDATION_COLLECTION', 'recommendation')
RECOMMENDS_STORAGE_MONGODB_FSYNC = getattr(settings, 'RECOMMENDS_STORAGE_MONGODB_FSYNC', False)



import logging
import math
import pymongo
from recommends.models import MockModel, MockSimilarity
from recommends.storages.base import BaseRecommendationStorage
from recommends.settings import RECOMMENDS_LOGGER_NAME, RECOMMENDS_STORAGE_LOGGING_THRESHOLD
from .settings import (
    RECOMMENDS_STORAGE_MONGODB_DATABASE,
    RECOMMENDS_STORAGE_MONGODB_SIMILARITY_COLLECTION,
    RECOMMENDS_STORAGE_MONGODB_RECOMMENDATION_COLLECTION,
    RECOMMENDS_STORAGE_MONGODB_FSYNC
)
from .managers import MongoStorageManager


logger = logging.getLogger(RECOMMENDS_LOGGER_NAME)


class MongoStorage(BaseRecommendationStorage):
    manager = MongoStorageManager()

    def _get_mock_models(self, spec, collection_name, limit, raw_id, mock_class=MockModel):
        connection = pymongo.Connection(RECOMMENDS_STORAGE_MONGODB_DATABASE['HOST'], RECOMMENDS_STORAGE_MONGODB_DATABASE['PORT'])
        db = connection[RECOMMENDS_STORAGE_MONGODB_DATABASE['NAME']]
        collection = db[collection_name]

        documents = collection.find(spec, limit=limit, sort=[('score', pymongo.DESCENDING)])
        if raw_id:
            if mock_class is MockModel:
                return [{
                        'object_id': item['object_id'],
                        'contect_type_id': item['object_ctype']}
                        for item in documents]
            elif mock_class is MockSimilarity:
                return [{
                    'related_object_id': item['related_object_id'],
                    'contect_type_id': item['object_ctype']}
                    for item in documents]
        return map(lambda x: mock_class(**x), documents)

    def get_similarities_for_object(self, obj, limit=10, raw_id=False):
        object_site_id = self.settings.SITE_ID
        spec = dict(related_object_site=object_site_id, **self.manager.filter_for_object(obj))
        collection_name = RECOMMENDS_STORAGE_MONGODB_SIMILARITY_COLLECTION

        return self._get_mock_models(spec, collection_name, limit, raw_id, mock_class=MockSimilarity)

    def get_recommendations_for_user(self, user, limit=10, raw_id=False):
        spec = {'user': user.id, 'object_site': self.settings.SITE_ID}
        collection_name = RECOMMENDS_STORAGE_MONGODB_RECOMMENDATION_COLLECTION

        return self._get_mock_models(spec, collection_name, limit, raw_id)

    def get_votes(self):
        pass

    def store_votes(self, iterable):
        pass

    def store_similarities(self, itemMatch):
        connection = pymongo.Connection(RECOMMENDS_STORAGE_MONGODB_DATABASE['HOST'], RECOMMENDS_STORAGE_MONGODB_DATABASE['PORT'])
        db = connection[RECOMMENDS_STORAGE_MONGODB_DATABASE['NAME']]
        collection = db[RECOMMENDS_STORAGE_MONGODB_SIMILARITY_COLLECTION]

        logger.info('saving similarities')
        count = 0

        for object_id, scores in itemMatch:
            object_target, object_target_site = self.resolve_identifier(object_id)

            for related_object_id, score in scores:
                if not math.isnan(score) and score > self.threshold_similarities:
                    object_related, object_related_site = self.resolve_identifier(related_object_id)
                    if object_target != object_related:
                        spec = self.manager.similarity_for_objects(object_target=object_target, object_target_site=object_target_site, object_related=object_related, object_related_site=object_related_site)
                        collection.update(spec, {'$set': {'score': score}}, upsert=True, fsync=RECOMMENDS_STORAGE_MONGODB_FSYNC)
                        count = count + 1

                        if count % RECOMMENDS_STORAGE_LOGGING_THRESHOLD == 0:
                            logger.debug('saved %s similarities...' % count)

        logger.info('saved %s similarities...' % count)

    def store_recommendations(self, recommendations):
        connection = pymongo.Connection(RECOMMENDS_STORAGE_MONGODB_DATABASE['HOST'], RECOMMENDS_STORAGE_MONGODB_DATABASE['PORT'])
        db = connection[RECOMMENDS_STORAGE_MONGODB_DATABASE['NAME']]
        collection = db[RECOMMENDS_STORAGE_MONGODB_RECOMMENDATION_COLLECTION]

        logger.info('saving recommendation')
        count = 0

        for (user, rankings) in recommendations:
            for object_id, score in rankings:
                if not math.isnan(score) and score > self.threshold_recommendations:
                    count = count + 1
                    object_recommended, site = self.resolve_identifier(object_id)
                    spec = self.manager.suggestion_for_object(
                        user=user,
                        object_recommended=object_recommended,
                        object_site=site
                    )
                    collection.update(spec, {'$set': {'score': score}}, upsert=True, fsync=RECOMMENDS_STORAGE_MONGODB_FSYNC)

                    if count % RECOMMENDS_STORAGE_LOGGING_THRESHOLD == 0:
                        logger.debug('saved %s recommendations...' % count)
        logger.info('saved %s recommendation...' % count)

    def remove_recommendations(self, obj):
        connection = pymongo.Connection(RECOMMENDS_STORAGE_MONGODB_DATABASE['HOST'], RECOMMENDS_STORAGE_MONGODB_DATABASE['PORT'])
        db = connection[RECOMMENDS_STORAGE_MONGODB_DATABASE['NAME']]
        collection = db[RECOMMENDS_STORAGE_MONGODB_RECOMMENDATION_COLLECTION]
        collection.remove(self.manager.filter_for_object(obj), fsync=RECOMMENDS_STORAGE_MONGODB_FSYNC)

    def remove_similarities(self, obj):
        connection = pymongo.Connection(RECOMMENDS_STORAGE_MONGODB_DATABASE['HOST'], RECOMMENDS_STORAGE_MONGODB_DATABASE['PORT'])
        db = connection[RECOMMENDS_STORAGE_MONGODB_DATABASE['NAME']]

        collection = db[RECOMMENDS_STORAGE_MONGODB_SIMILARITY_COLLECTION]

        collection.remove(self.manager.filter_for_object(obj), fsync=RECOMMENDS_STORAGE_MONGODB_FSYNC)
        collection.remove(self.manager.filter_for_related_object(obj), fsync=RECOMMENDS_STORAGE_MONGODB_FSYNC)



from math import sqrt


@staticmethod
def sim_distance(p1, p2):
    """Returns a distance-based similarity score for p1 and p2"""
    # Get the list of shared_items
    si = [item for item in p1 if item in p2]

    if len(si) != 0:
        squares = [pow(p1[item] - p2[item], 2) for item in si]
        # Add up the squares of all the differences
        sum_of_squares = sum(squares)
        return 1 / (1 + sqrt(sum_of_squares))
    return 0


@staticmethod
def sim_pearson(p1, p2):
    """
    Returns the Pearson correlation coefficient for p1 and p2
    """
    # Get the list of mutually rated items
    si = [item for item in p1 if item in p2]

    # Find the number of elements
    n = len(si)

    # if they have no ratings in common, return 0
    if n != 0:
        # Add up all the preferences
        sum1 = sum([p1[it] for it in si])
        sum2 = sum([p2[it] for it in si])

        # Sum up the squares
        sum1Sq = sum([pow(p1[it], 2) for it in si])
        sum2Sq = sum([pow(p2[it], 2) for it in si])

        # Sum up the products
        pSum = sum([p1[it] * p2[it] for it in si])

        # Calculate Pearson score
        num = pSum - (sum1 * sum2 / n)
        den = sqrt((sum1Sq - pow(sum1, 2) / n) * (sum2Sq - pow(sum2, 2) / n))
        if den == 0:
            return 0
        r = num / den
        return r
    return 0



from __future__ import unicode_literals
from django.db import models
from django.contrib.sites.models import Site
from django.contrib.auth import models as auth_models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class RecProduct(models.Model):
    """A generic Product"""
    name = models.CharField(blank=True, max_length=100)
    sites = models.ManyToManyField(Site)

    class Meta:
        app_label = 'recommends'

    def __str__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('product_detail', [self.id])

    def sites_str(self):
        return ', '.join([s.name for s in self.sites.all()])
    sites_str.short_description = 'sites'


@python_2_unicode_compatible
class RecVote(models.Model):
    """A Vote on a Product"""
    user = models.ForeignKey(auth_models.User, related_name='rec_votes')
    product = models.ForeignKey(RecProduct)
    site = models.ForeignKey(Site)
    score = models.FloatField()

    class Meta:
        app_label = 'recommends'

    def __str__(self):
        return "Vote"



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoorm', '0001_initial'),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='recommendation',
            name='object_ctype',
            field=models.ForeignKey(to='contenttypes.ContentType'),
        ),
        migrations.AlterField(
            model_name='similarity',
            name='object_ctype',
            field=models.ForeignKey(to='contenttypes.ContentType'),
        ),
        migrations.AlterField(
            model_name='similarity',
            name='related_object_ctype',
            field=models.ForeignKey(related_name='similar', to='contenttypes.ContentType'),
        ),
    ]



from recommends.utils import ctypes_dict


class CachedContentTypesMixin(object):
    _ctypes = None

    @property
    def ctypes(self):
        if self._ctypes is None:
            self._ctypes = ctypes_dict()
        return self._ctypes

    def get_ctype_id_for_obj(self, obj):
        app_label = obj._meta.app_label
        module_name = obj._meta.model_name
        return self.ctypes["%s.%s" % (app_label, module_name)]


class DictStorageManager(CachedContentTypesMixin):
    def similarity_for_objects(self, object_target, object_target_site, object_related, object_related_site):
        object_ctype_id = self.get_ctype_id_for_obj(object_target)
        object_id = object_target.id

        related_object_ctype_id = self.get_ctype_id_for_obj(object_related)
        related_object_id = object_related.id

        return dict(
            object_ctype=object_ctype_id,
            object_id=object_id,
            object_site=object_target_site.id,
            related_object_ctype=related_object_ctype_id,
            related_object_id=related_object_id,
            related_object_site=object_related_site.id,
        )

    def suggestion_for_object(self, user, object_recommended, object_site):
        object_ctype_id = self.get_ctype_id_for_obj(object_recommended)
        object_id = object_recommended.id

        return dict(
            object_ctype=object_ctype_id,
            object_id=object_id,
            object_site=object_site.id,
            user=user.id,
        )



import contextlib
import errno
import os
import time
import tempfile
import importlib


def import_from_classname(class_name_str):
    module, class_name = class_name_str.rsplit('.', 1)
    Class = getattr(importlib.import_module(module), class_name)
    return Class


def ctypes_dict():
    from django.contrib.contenttypes.models import ContentType

    values = ContentType.objects.values_list('app_label', 'model', 'id')
    ctypes = {}
    [ctypes.update({"%s.%s" % x[:2]: x[2]}) for x in values]
    return ctypes


@contextlib.contextmanager
def filelock(name, wait_delay=.1):
    path = os.path.join(tempfile.gettempdir(), name)
    while True:
        try:
            fd = os.open(path, os.O_CREAT | os.O_EXCL | os.O_RDWR)
        except OSError as e:
            if e.errno != errno.EEXIST:
                raise
            time.sleep(wait_delay)
            continue
        else:
            break
    try:
        yield fd
    finally:
        os.close(fd)
        os.unlink(path)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Recommendation',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_ctype', models.PositiveIntegerField()),
                ('object_id', models.PositiveIntegerField()),
                ('object_site', models.PositiveIntegerField()),
                ('user', models.PositiveIntegerField()),
                ('score', models.FloatField(default=None, null=True, blank=True)),
            ],
            options={
                'ordering': ['-score'],
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Similarity',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('object_ctype', models.PositiveIntegerField()),
                ('object_id', models.PositiveIntegerField()),
                ('object_site', models.PositiveIntegerField()),
                ('score', models.FloatField(default=None, null=True, blank=True)),
                ('related_object_ctype', models.PositiveIntegerField()),
                ('related_object_id', models.PositiveIntegerField()),
                ('related_object_site', models.PositiveIntegerField()),
            ],
            options={
                'ordering': ['-score'],
                'verbose_name_plural': 'similarities',
            },
            bases=(models.Model,),
        ),
        migrations.AlterUniqueTogether(
            name='similarity',
            unique_together=set([('object_ctype', 'object_id', 'object_site', 'related_object_ctype', 'related_object_id', 'related_object_site')]),
        ),
        migrations.AlterUniqueTogether(
            name='recommendation',
            unique_together=set([('object_ctype', 'object_id', 'user')]),
        ),
    ]



from django.conf import settings

RECOMMENDS_STORAGE_DATABASE_ALIAS = getattr(settings, 'RECOMMENDS_STORAGE_DATABASE_ALIAS', 'recommends')
RECOMMENDS_STORAGE_COMMIT_THRESHOLD = getattr(settings, 'RECOMMENDS_STORAGE_COMMIT_THRESHOLD', 1000)



from collections import defaultdict
from django.apps import apps


def model_path(obj):
    return '%s.%s' % (obj._meta.app_label, obj._meta.object_name.lower())


class IdentifierManager(object):
    _sites = None
    _ctypes = None

    @property
    def sites(self):
        if self._sites is None:
            from django.contrib.sites.models import Site

            self._sites = dict([(s.id, s) for s in Site.objects.all()])
        return self._sites

    @property
    def ctypes(self):
        if self._ctypes is None:
            from django.contrib.contenttypes.models import ContentType

            self._ctypes = dict([("%s.%s" % (c.app_label, c.model), c) for c in ContentType.objects.all()])
        return self._ctypes

    def resolve_identifier(self, identifier):
        """
        The opposite of ``get_identifier()``
        """
        app_module, site_id, object_id = identifier.split(':')
        app_label, model = app_module.split('.')
        site = self.sites[int(site_id)]
        ModelClass = apps.get_model(app_label, model)
        model = ModelClass.objects.get(pk=object_id)
        return model, site

    def identifier_to_dict(self, identifier, score=None, related=False):
        """
        The opposite of ``get_identifier()``
        """
        app_module, site_id, object_id = identifier.split(':')
        ctype = self.ctypes[app_module]

        if related:
            spec = {
                'related_object_ctype': ctype.id,
                'related_object_id': int(object_id),
                'related_object_site': int(site_id)
            }
        else:
            spec = {
                'object_ctype': ctype.id,
                'object_id': int(object_id),
                'object_site': int(site_id)
            }
        if score is not None:
            spec['score'] = score

        return spec

    def get_identifier(self, obj, site_id):
        """
        Given a Django Model, returns a string identifier in the format
        <app_label>.<model>:<site_id>:<object_id>.
        """
        return "%s:%s:%s" % (model_path(obj), site_id, obj.id)


def convert_vote_list_to_userprefs(vote_list):
    """
    Return a user-centerd prefernce matrix.

    ``vote_list must be`` composed of (user_id, object_identifier, rating)

    ``object_identifier`` is any string that uniquely identifies the object ie:
    <app_label>.<model>:<object_id>.

    The ``utils.get_identifier`` method is provided as convenience for creating such identifiers.
    """
    prefs = defaultdict(dict)
    for pref in vote_list:
        prefs[pref[0]][pref[1]] = pref[2]
    return prefs


def convert_vote_list_to_itemprefs(vote_list):
    """
    Return a item-centerd prefernce matrix.

    ``vote_list must be`` composed of (user_id, object_identifier, rating)

    ``object_identifier`` is any string that uniquely identifies the object ie:
    <app_label>.<model>:<object_id>.

    The ``utils.get_identifier`` method is provided as convenience for creating such identifiers.
    """
    prefs = defaultdict(dict)
    for pref in vote_list:
        prefs[pref[1]][pref[0]] = pref[2]
    return prefs


def similary_results_to_itemMatch(qs, provider):
    itemMatch = defaultdict(list)
    for i in qs:
        site = i.related_object_site
        item = provider.get_identifier(i.get_object(), site)
        similarity = i.score
        item2 = provider.get_identifier(i.get_related_object(), site)

        itemMatch[item].append((similarity, item2))

    return itemMatch



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Similarity'
        db.create_table(u'djangoorm_similarity', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('object_ctype', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('object_site', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('score', self.gf('django.db.models.fields.FloatField')(default=None, null=True, blank=True)),
            ('related_object_ctype', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('related_object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('related_object_site', self.gf('django.db.models.fields.PositiveIntegerField')()),
        ))
        db.send_create_signal(u'djangoorm', ['Similarity'])

        # Adding unique constraint on 'Similarity', fields ['object_ctype', 'object_id', 'object_site', 'related_object_ctype', 'related_object_id', 'related_object_site']
        db.create_unique(u'djangoorm_similarity', ['object_ctype', 'object_id', 'object_site', 'related_object_ctype', 'related_object_id', 'related_object_site'])

        # Adding model 'Recommendation'
        db.create_table(u'djangoorm_recommendation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('object_ctype', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('object_site', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('user', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('score', self.gf('django.db.models.fields.FloatField')(default=None, null=True, blank=True)),
        ))
        db.send_create_signal(u'djangoorm', ['Recommendation'])

        # Adding unique constraint on 'Recommendation', fields ['object_ctype', 'object_id', 'user']
        db.create_unique(u'djangoorm_recommendation', ['object_ctype', 'object_id', 'user'])


    def backwards(self, orm):
        # Removing unique constraint on 'Recommendation', fields ['object_ctype', 'object_id', 'user']
        db.delete_unique(u'djangoorm_recommendation', ['object_ctype', 'object_id', 'user'])

        # Removing unique constraint on 'Similarity', fields ['object_ctype', 'object_id', 'object_site', 'related_object_ctype', 'related_object_id', 'related_object_site']
        db.delete_unique(u'djangoorm_similarity', ['object_ctype', 'object_id', 'object_site', 'related_object_ctype', 'related_object_id', 'related_object_site'])

        # Deleting model 'Similarity'
        db.delete_table(u'djangoorm_similarity')

        # Deleting model 'Recommendation'
        db.delete_table(u'djangoorm_recommendation')


    models = {
        u'djangoorm.recommendation': {
            'Meta': {'ordering': "[u'-score']", 'unique_together': "((u'object_ctype', u'object_id', u'user'),)", 'object_name': 'Recommendation'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_ctype': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_site': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.FloatField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        u'djangoorm.similarity': {
            'Meta': {'ordering': "[u'-score']", 'unique_together': "((u'object_ctype', u'object_id', u'object_site', u'related_object_ctype', u'related_object_id', u'related_object_site'),)", 'object_name': 'Similarity'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_ctype': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_site': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'related_object_ctype': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'related_object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'related_object_site': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.FloatField', [], {'default': 'None', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['djangoorm']


import importlib

from django.apps import AppConfig, apps
from .settings import RECOMMENDS_AUTODISCOVER_MODULE


class RecommendsConfig(AppConfig):
    name = 'recommends'

    def ready(self):
        if not RECOMMENDS_AUTODISCOVER_MODULE:
            return

        for appconfig in apps.get_app_configs():
            try:
                importlib.import_module('.' + RECOMMENDS_AUTODISCOVER_MODULE, appconfig.name)
            except ImportError:
                pass



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):

        # Renaming column for 'Similarity.related_object_ctype' to match new field type.
        db.rename_column(u'djangoorm_similarity', 'related_object_ctype', 'related_object_ctype_id')
        # Changing field 'Similarity.related_object_ctype'
        db.alter_column(u'djangoorm_similarity', 'related_object_ctype_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType']))
        # Adding index on 'Similarity', fields ['related_object_ctype']
        db.create_index(u'djangoorm_similarity', ['related_object_ctype_id'])


        # Renaming column for 'Similarity.object_ctype' to match new field type.
        db.rename_column(u'djangoorm_similarity', 'object_ctype', 'object_ctype_id')
        # Changing field 'Similarity.object_ctype'
        db.alter_column(u'djangoorm_similarity', 'object_ctype_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType']))
        # Adding index on 'Similarity', fields ['object_ctype']
        db.create_index(u'djangoorm_similarity', ['object_ctype_id'])


        # Renaming column for 'Recommendation.object_ctype' to match new field type.
        db.rename_column(u'djangoorm_recommendation', 'object_ctype', 'object_ctype_id')
        # Changing field 'Recommendation.object_ctype'
        db.alter_column(u'djangoorm_recommendation', 'object_ctype_id', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType']))
        # Adding index on 'Recommendation', fields ['object_ctype']
        db.create_index(u'djangoorm_recommendation', ['object_ctype_id'])


    def backwards(self, orm):
        # Removing index on 'Recommendation', fields ['object_ctype']
        db.delete_index(u'djangoorm_recommendation', ['object_ctype_id'])

        # Removing index on 'Similarity', fields ['object_ctype']
        db.delete_index(u'djangoorm_similarity', ['object_ctype_id'])

        # Removing index on 'Similarity', fields ['related_object_ctype']
        db.delete_index(u'djangoorm_similarity', ['related_object_ctype_id'])


        # Renaming column for 'Similarity.related_object_ctype' to match new field type.
        db.rename_column(u'djangoorm_similarity', 'related_object_ctype_id', 'related_object_ctype')
        # Changing field 'Similarity.related_object_ctype'
        db.alter_column(u'djangoorm_similarity', 'related_object_ctype', self.gf('django.db.models.fields.PositiveIntegerField')())

        # Renaming column for 'Similarity.object_ctype' to match new field type.
        db.rename_column(u'djangoorm_similarity', 'object_ctype_id', 'object_ctype')
        # Changing field 'Similarity.object_ctype'
        db.alter_column(u'djangoorm_similarity', 'object_ctype', self.gf('django.db.models.fields.PositiveIntegerField')())

        # Renaming column for 'Recommendation.object_ctype' to match new field type.
        db.rename_column(u'djangoorm_recommendation', 'object_ctype_id', 'object_ctype')
        # Changing field 'Recommendation.object_ctype'
        db.alter_column(u'djangoorm_recommendation', 'object_ctype', self.gf('django.db.models.fields.PositiveIntegerField')())

    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'djangoorm.recommendation': {
            'Meta': {'ordering': "[u'-score']", 'unique_together': "((u'object_ctype', u'object_id', u'user'),)", 'object_name': 'Recommendation'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_ctype': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_site': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.FloatField', [], {'default': 'None', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        u'djangoorm.similarity': {
            'Meta': {'ordering': "[u'-score']", 'unique_together': "((u'object_ctype', u'object_id', u'object_site', u'related_object_ctype', u'related_object_id', u'related_object_site'),)", 'object_name': 'Similarity'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_ctype': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'object_site': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'related_object_ctype': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "u'similar'", 'to': u"orm['contenttypes.ContentType']"}),
            'related_object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'related_object_site': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'score': ('django.db.models.fields.FloatField', [], {'default': 'None', 'null': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['djangoorm']


#!/usr/bin/env python
import sys
from os import path

import django
from django.test.utils import get_runner

from django.conf import settings

PROJECT_DIR = path.dirname(path.realpath(__file__))


settings.configure(
    DATABASES={
        'default': {'ENGINE': 'django.db.backends.sqlite3'}
    },
    INSTALLED_APPS=[
        'django.contrib.auth',
        'django.contrib.sessions',
        'django.contrib.contenttypes',
        'django.contrib.sites',
        'recommends',
        'recommends.storages.djangoorm',
        'recommends.tests',
    ],
    ROOT_URLCONF='recommends.tests.urls',
    TEMPLATE_DIRS=(
        path.join(PROJECT_DIR, 'templates'),
    ),
    MIDDLEWARE_CLASSES=(
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
    ),
    BROKER_URL='redis://localhost:6379/0',
    CELERY_ALWAYS_EAGER=True,
    ALLOWED_HOSTS=['*'],
    SITE_ID=1,
    RECOMMENDS_TEST_REDIS=True,
    RECOMMENDS_TEST_MONGO=True,
    RECOMMENDS_TEST_RECSYS=True,
    RECOMMENDS_STORAGE_MONGODB_FSYNC=True,
    RECOMMENDS_TASK_RUN=True,
)


def runtests(*test_args):
    try:
        django.setup()  # Django 1.7+
    except AttributeError:
        pass
    runner_class = get_runner(settings)
    test_runner = runner_class(verbosity=1, interactive=True)
    failures = test_runner.run_tests(['recommends'])
    sys.exit(failures)

if __name__ == '__main__':
    runtests()
